#ifndef __DCPJPEG_H__
#define __DCPJPEG_H__

/* #includes */
#include <setjmp.h>
#include "jpeglib.h"


#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
typedef struct _tagDcpJpeg;
typedef struct _tagCpJpeg;
#endif

/*  Structures aggrandies pour retrouver nos billes
    qd on est appele par les callbacks de la jpeglib  */
typedef struct _tagjpeg_dc_struct_ext	/*  Decompression  */
{
    struct jpeg_decompress_struct djpeg;
    struct _tagDcpJpeg *pDcpJpeg;
} jpeg_dc_struct_ext;

typedef struct _tagjpeg_cp_struct_ext	/*  Compression  */
{
    struct jpeg_compress_struct cjpeg;
    struct _tagCpJpeg *pCpJpeg;
} jpeg_cp_struct_ext;

typedef struct _tagjpeg_error_mgr_ext	/*  Erreur  */
{
    struct jpeg_error_mgr jerr;
    jmp_buf	jmp_env;	/* Gestion des erreurs */
    void * lParam;		/*  En general, pointeur sur structure mere  */
    void (*ErrorMessage)( void * lParam, char * pszBuffer );

    union {
	struct _tagDcpJpeg *pDcpJpeg;
	struct _tagCpJpeg *pCpJpeg;
    } UnionCpDcp;

} jpeg_error_mgr_ext;


/*  Structure gerant la decompression  */
typedef struct _tagDcpJpeg
{
    jpeg_dc_struct_ext djpegext;
    jpeg_error_mgr_ext jerrext;
    int	nState;

    char	*pBuffer;	/*  Donnees decompressees  */
    int	cbBuffer;	/*  Taille du buffer	*/

    char	*pSaveBuffer;	/*  Buffer de sauvegarde  */
    int	cbSaveBuffer;	/*  Taille du buffer	*/
    int	cbSaveLast;	/*  Quantite actuelle	*/
    int	nOffset;	/*  = recul a effectuer	*/
    int	cbToSkip;

    int	fOnePass;	/* Decompression en une passe */
    int	fDataGiven;	/* Donnees deja donnees */

    /*  Gestion chgt d'echelle (soit num et denom, soit output, pas les 2)*/
    int scale_num;		/*  Numerateur pour l'echelle  */
    int scale_denom;	/*  Denominateur pour l'echelle */
    int output_width;	/*  Largeur desiree */


    void * lParam;		/*  En general, pointeur sur structure mere  */
    int (*GetData)(void *, char **, int *);	/*  Fonction utilisee pour extraire les donnees  */
} DcpJpeg;



/*  Structure gerant la compression  */
typedef struct _tagCpJpeg
{
    jpeg_cp_struct_ext cjpegext;
    jpeg_error_mgr_ext jerrext;
    int	nState;

    char	*pBuffer;	/*  Donnees compressees  */
    int	cbBuffer;	/*  Taille du buffer	 */
    int	nLineDones;	/*  Nombre de lignes ok  */

    int nQuality;		/*  Qualite de 0 a 100   */

    void * lParam;		/*  En general, pointeur sur structure mere  */
    int (*WriteData)(void *, char *, int);	/*  Fonction utilisee pour ecrire les donnees  */

    jmp_buf	jmp_env;	/* Gestion des erreurs */
} CpJpeg;


/*  Protos decompression */
DcpJpeg * DcpJpegInit( int (*pfGetData)(void *, char **, int *), void * lParam, void (*pfErrorMessage)( void *, char * ) );
DcpJpeg * DcpJpegInit2( int (*pfGetData)(void *, char **, int *), void * lParam, void (*pfErrorMessage)( void *, char * ), int fOnePass );
void DcpJpegInitScale( DcpJpeg * pDcpJpeg, int scale_num, int scale_denom, int output_width );
void DcpJpegDestroy( DcpJpeg * pDcpJpeg );
void DcpJpegGetParam( DcpJpeg * pDcpJpeg, unsigned int * , unsigned int *,int *, int * );
int DcpJpegDecode( DcpJpeg * pDcpJpeg, char **ppDecompress, int * pcbDecompress );

/*  Protos compression */
CpJpeg * CpJpegInit( int (*pfWriteData)(void *, char *, int ), void * lParam, void (*pfErrorMessage)( void *, char * ) );
void CpJpegDestroy( CpJpeg * pCpJpeg );
void CpJpegSetParam( CpJpeg * pCpJpeg, unsigned int image_width, 
	unsigned int image_height,int input_components, int in_color_space, int nQuality );
int CpJpegEncode( CpJpeg * pCpJpeg, char *pData, int cbData );

/*  Programme d'exemple  */
void TestJpeg( char *pszFileSource, char *pszFileDest, int nQuality, int fOnePass );


#ifdef __cplusplus
}
#endif


#endif	/*__DCPJPEG_H__*/
